@@ -9,11 +9,12 @@
 use MediaWiki\Logger\LoggerFactory;
 
 class MathHooks {
-	const mathCacheKey = 'math=';
+	private static $tags = [];
+	const MATHCACHEKEY = 'math=';
 
-	public static function mathConstantToString( $value, array $defs, $prefix, $default ){
+	public static function mathConstantToString( $value, array $defs, $prefix, $default ) {
 		foreach ( $defs as $defKey => $defValue ) {
-			if( !defined( $defKey ) ) {
+			if ( !defined( $defKey ) ) {
 				define( $defKey, $defValue );
 			} elseif ( $defValue !== constant( $defKey ) ) {
 				throw new Exception( 'Math constant "'. $defKey . '" has unexpected value "' .
@@ -21,15 +22,15 @@ class MathHooks {
 			}
 		}
 		$invDefs = array_flip( $defs );
-		if ( is_int( $value ) ){
-			if ( array_key_exists( $value , $invDefs ) ) {
+		if ( is_int( $value ) ) {
+			if ( array_key_exists( $value, $invDefs ) ) {
 				$value = $invDefs[$value];
 			} else {
 				return $default;
 			}
 		}
-		if ( is_string( $value ) ){
-			$newValues = array();
+		if ( is_string( $value ) ) {
+			$newValues = [];
 			foreach ( $defs as $k => $v ) {
 				$newValues[$k] = preg_replace_callback( '/_(.)/', function ( $matches ) {
 					return strtoupper( $matches[1] );
@@ -37,7 +38,7 @@ class MathHooks {
 			}
 			if ( array_key_exists( $value, $defs ) ) {
 				return $newValues[$value];
-			} elseif (in_array( $value, $newValues) ){
+			} elseif ( in_array( $value, $newValues ) ) {
 				return $value;
 			}
 		}
@@ -45,49 +46,49 @@ class MathHooks {
 	}
 
 	public static function mathStyleToString( $style, $default = 'inlineDisplaystyle' ) {
-		$defs = array (
+		$defs = [
 			'MW_MATHSTYLE_INLINE_DISPLAYSTYLE'  => 0, // default large operator inline
 			'MW_MATHSTYLE_DISPLAY'              => 1, // large operators centered in a new line
 			'MW_MATHSTYLE_INLINE'               => 2, // small operators inline
 			'MW_MATHSTYLE_LINEBREAK'            => 3, // break long lines (experimental)
-		);
+		];
 		return self::mathConstantToString( $style, $defs, $prefix = 'MW_MATHSTYLE_', $default );
 	}
 
 	public static function mathCheckToString( $style, $default = 'always' ) {
-		$defs = array (
+		$defs = [
 			'MW_MATH_CHECK_ALWAYS' => 0,
 			'MW_MATH_CHECK_NEVER'  => 1,
 			'MW_MATH_CHECK_NEW'    => 2,
-		);
+		];
 		return self::mathConstantToString( $style, $defs, $prefix = 'MW_MATH_CHECK_', $default );
 	}
 
 	public static function mathModeToString( $mode, $default = 'png' ) {
-//		The following deprecated modes have been removed:
-//		  'MW_MATH_SIMPLE'      => 1
-//		  'MW_MATH_HTML'        => 2
-//		  'MW_MATH_MODERN'      => 4
-//		  'MW_MATH_MATHJAX'     => 6
-//		  'MW_MATH_LATEXML_JAX' => 8
+		// The following deprecated modes have been removed:
+		// 'MW_MATH_SIMPLE'      => 1
+		// 'MW_MATH_HTML'        => 2
+		// 'MW_MATH_MODERN'      => 4
+		// 'MW_MATH_MATHJAX'     => 6
+		// 'MW_MATH_LATEXML_JAX' => 8
 
-		$defs = array (
+		$defs = [
 			'MW_MATH_PNG'    => 0,
 			'MW_MATH_SOURCE' => 3,
 			'MW_MATH_MATHML' => 5,
-			'MW_MATH_LATEXML'=> 7 );
+			'MW_MATH_LATEXML'=> 7 ];
 
 		return self::mathConstantToString( $mode, $defs, $prefix = 'MW_MATH_', $default );
 	}
 
 	public static function mathModeToHashKey( $mode, $default = 0 ) {
-		$defs = array (
+		$defs = [
 			'png'    => 0,
 			'source' => 3,
 			'mathml' => 5,
-			'latexml'=> 7 );
+			'latexml'=> 7 ];
 
-		if ( array_key_exists( $mode, $defs ) ){
+		if ( array_key_exists( $mode, $defs ) ) {
 			return $defs[$mode];
 		} else {
 			return $default;
@@ -101,12 +102,12 @@ class MathHooks {
 	 * @param User $user reference to the current user
 	 * @param array &$forOptions userOptions used on that page
 	 */
-	public static function onPageRenderingHash( &$confstr, $user = false, &$forOptions = array() ) {
+	public static function onPageRenderingHash( &$confstr, $user = false, &$forOptions = [] ) {
 		global $wgUser;
 
 		// To be independent of the MediaWiki core version,
 		// we check if the core caching logic for math is still available.
-		if ( ! is_callable( 'ParserOptions::getMath' ) && in_array( 'math', $forOptions ) ) {
+		if ( !is_callable( 'ParserOptions::getMath' ) && in_array( 'math', $forOptions ) ) {
 			if ( $user === false ) {
 				$user = $wgUser;
 			}
@@ -116,19 +117,20 @@ class MathHooks {
 			// Check if the key already contains the math option part
 			if (
 				!preg_match(
-					'/(^|!)' . self::mathCacheKey . $mathOption . '(!|$)/',
+					'/(^|!)' . self::MATHCACHEKEY . $mathOption . '(!|$)/',
 					$confstr
 				)
 			) {
-				// The math part of cache key starts with "math=" followed by a star or a number for the math mode
-				if ( preg_match( '/(^|!)' . self::mathCacheKey . '[*\d]m?(!|$)/', $confstr ) ) {
+				// The math part of cache key starts with "math="
+				// followed by a star or a number for the math mode
+				if ( preg_match( '/(^|!)' . self::MATHCACHEKEY . '[*\d]m?(!|$)/', $confstr ) ) {
 					$confstr = preg_replace(
-						'/(^|!)' . self::mathCacheKey . '[*\d]m?(!|$)/',
-						'\1' . self::mathCacheKey . $mathOption . '\2',
+						'/(^|!)' . self::MATHCACHEKEY . '[*\d]m?(!|$)/',
+						'\1' . self::MATHCACHEKEY . $mathOption . '\2',
 						$confstr
 					);
 				} else {
-					$confstr .= '!' . self::mathCacheKey . $mathOption;
+					$confstr .= '!' . self::MATHCACHEKEY . $mathOption;
 				}
 
 				LoggerFactory::getInstance( 'Math' )->debug( "New cache key: $confstr" );
@@ -163,7 +165,8 @@ class MathHooks {
 	 * @return Boolean: true
 	 */
 	static function onParserFirstCallInit( $parser ) {
-		$parser->setHook( 'math', array( 'MathHooks', 'mathTagHook' ) );
+		$parser->setHook( 'math', [ 'MathHooks', 'mathTagHook' ] );
+		$parser->setHook( 'ce', [ 'MathHooks', 'ceTagHook' ] );
 
 		return true;
 	}
@@ -177,7 +180,7 @@ class MathHooks {
 	 * @return array
 	 */
 	static function mathTagHook( $content, $attributes, $parser ) {
-
+		static $n = 1;
 		if ( trim( $content ) === '' ) { // bug 8372
 			return '';
 		}
@@ -185,40 +188,55 @@ class MathHooks {
 		$mode = self::mathModeToString( $parser->getUser()->getOption( 'math' ) );
 		// Indicate that this page uses math.
 		// This affects the page caching behavior.
-		if ( is_callable( 'ParserOptions::getMath' ) ) {
-			$parser->getOptions()->getMath();
-		} else {
-			$parser->getOptions()->optionUsed( 'math' );
-		}
-
+		$parser->getOptions()->optionUsed( 'math' );
 		$renderer = MathRenderer::getRenderer( $content, $attributes, $mode );
 
-		$checkResult = $renderer->checkTex();
+		$parser->getOutput()->addModuleStyles( [ 'ext.math.styles' ] );
+		if ( $mode == 'mathml' ) {
+			$parser->getOutput()->addModules( [ 'ext.math.scripts' ] );
+			$marker = Parser::MARKER_PREFIX .
+				'-postMath-' . sprintf( '%08X', $n ++ ) .
+				Parser::MARKER_SUFFIX;
+			self::$tags[$marker] = [ $renderer, $parser ];
+			return $marker;
+		}
+		return [ self::mathPostTagHook( $renderer, $parser ), 'markerType' => 'nowiki' ];
+	}
+
+	/**
+	 * Callback function for the <math> parser hook.
+	 *
+	 * @param Parser $parser
+	 * @param MathRenderer $renderer
+	 * @return array
+	 * @throws FatalError
+	 * @throws MWException
+	 */
+	private static function mathPostTagHook( $renderer, $parser ) {
+		$checkResult = $renderer->checkTeX();
 
 		if ( $checkResult !== true ) {
-			// Returns the error message
+			// Returns the error message and add tracking category
+			$parser->addTrackingCategory( 'math-tracking-category-error' );
 			return $renderer->getLastError();
 		}
 
 		if ( $renderer->render() ) {
-			LoggerFactory::getInstance( 'Math' )->info( "Rendering successful. Writing output" );
+			LoggerFactory::getInstance( 'Math' )->debug( "Rendering successful. Writing output" );
 			$renderedMath = $renderer->getHtmlOutput();
 		} else {
 			LoggerFactory::getInstance( 'Math' )->warning(
 				"Rendering failed. Printing error message." );
+			$parser->addTrackingCategory( 'math-tracking-category-error' );
 			return $renderer->getLastError();
 		}
 		Hooks::run( 'MathFormulaPostRender',
-			array( $parser, &$renderer, &$renderedMath ) );// Enables indexing of math formula
-		$parser->getOutput()->addModuleStyles( array( 'ext.math.styles' ) );
-		if ( $mode == 'mathml' ) {
-			$parser->getOutput()->addModuleStyles( array( 'ext.math.desktop.styles' ) );
-			$parser->getOutput()->addModules( array( 'ext.math.scripts' ) );
-		}
+			[ $parser, $renderer, &$renderedMath ] );// Enables indexing of math formula
+
 		// Writes cache if rendering was successful
 		$renderer->writeCache();
 
-		return array( $renderedMath, "markerType" => 'nowiki' );
+		return $renderedMath;
 	}
 
 	/**
@@ -230,16 +248,16 @@ class MathHooks {
 	 */
 	static function onGetPreferences( $user, &$defaultPreferences ) {
 		global $wgDefaultUserOptions;
-		$defaultPreferences['math'] = array(
+		$defaultPreferences['math'] = [
 			'type' => 'radio',
 			'options' => array_flip( self::getMathNames() ),
 			'label' => '&#160;',
 			'section' => 'rendering/math',
-		);
+		];
 		// If the default option is not in the valid options the
 		// user interface throws an exception (BUG 64844)
 		$mode = MathHooks::mathModeToString( $wgDefaultUserOptions['math'] );
-		if ( ! in_array( $mode , MathRenderer::getValidModes()  ) ) {
+		if ( ! in_array( $mode, MathRenderer::getValidModes() ) ) {
 			LoggerFactory::getInstance( 'Math' )->error( 'Misconfiguration: '.
 				"\$wgDefaultUserOptions['math'] is not in " . MathRenderer::getValidModes() . ".\n".
 				"Please check your LocalSetting.php file." );
@@ -256,9 +274,9 @@ class MathHooks {
 	 * @return array of strings
 	 */
 	public static function getMathNames() {
-		$names = array();
-		foreach ( MathRenderer::getValidModes()  as $mode ) {
-			$names[ $mode ] = wfMessage( 'mw_math_' . $mode )->escaped();
+		$names = [];
+		foreach ( MathRenderer::getValidModes() as $mode ) {
+			$names[$mode] = wfMessage( 'mw_math_' . $mode )->escaped();
 		}
 
 		return $names;
@@ -292,7 +310,7 @@ class MathHooks {
 			throw new Exception( 'Math extension is only necessary in 1.18 or above' );
 		}
 
-		$map = array( 'mysql', 'sqlite', 'postgres', 'oracle', 'mssql' );
+		$map = [ 'mysql', 'sqlite', 'postgres', 'oracle', 'mssql' ];
 
 		$type = $updater->getDB()->getType();
 
@@ -302,10 +320,10 @@ class MathHooks {
 		$sql = __DIR__ . '/db/math.' . $type . '.sql';
 		$updater->addExtensionTable( 'math', $sql );
 		if ( in_array( 'latexml', MathRenderer::getValidModes() ) ) {
-			if ( in_array( $type, array( 'mysql', 'sqlite', 'postgres' ) ) ) {
+			if ( in_array( $type, [ 'mysql', 'sqlite', 'postgres' ] ) ) {
 				$sql = __DIR__ . '/db/mathlatexml.' . $type . '.sql';
 				$updater->addExtensionTable( 'mathlatexml', $sql );
-				if ( $type == 'mysql' ){
+				if ( $type == 'mysql' ) {
 					$sql = __DIR__ . '/db/patches/mathlatexml.mathml-length-adjustment.mysql.sql';
 					$updater->modifyExtensionField( 'mathlatexml', 'math_mathml', $sql );
 				}
@@ -313,8 +331,8 @@ class MathHooks {
 				throw new Exception( "Math extension does not currently support $type database for LaTeXML." );
 			}
 		}
-		if ( in_array( 'mathml', MathRenderer::getValidModes()  ) ) {
-			if ( in_array( $type, array( 'mysql', 'sqlite', 'postgres' ) ) ) {
+		if ( in_array( 'mathml', MathRenderer::getValidModes() ) ) {
+			if ( in_array( $type, [ 'mysql', 'sqlite', 'postgres' ] ) ) {
 				$sql = __DIR__ . '/db/mathoid.' . $type . '.sql';
 				$updater->addExtensionTable( 'mathoid', $sql );
 			} else {
@@ -339,18 +357,32 @@ class MathHooks {
 	}
 
 	/**
-	 * Links to the unit test files for the test cases.
-	 *
-	 * @param string $files
-	 * @return boolean (true)
+	 * @param Parser $parser
+	 * @param $text
+	 * @return bool
 	 */
-	static function onRegisterUnitTests( &$files ) {
-		$testDir = __DIR__ . '/tests/';
-		$files = array_merge( $files, glob( "$testDir/*Test.php" ) );
-
+	public static function onParserAfterTidy( &$parser, &$text ) {
+		$rbis = [];
+		foreach ( self::$tags as $key => $tag ){
+			/** @var MathRenderer $renderer */
+			$renderer = $tag[0];
+			$rbi = new MathRestbaseInterface( $renderer->getTex(), $renderer->getInputType() );
+			$renderer->setRestbaseInterface( $rbi );
+			$rbis[] = $rbi;
+		}
+		MathRestbaseInterface::batchEvaluate( $rbis );
+		foreach ( self::$tags as $key => $tag ){
+			$value = call_user_func_array( [ "MathHooks","mathPostTagHook" ], $tag );
+			// Workaround for https://phabricator.wikimedia.org/T103269
+			$text = preg_replace( '/(<mw:editsection[^>]*>.*?)' . preg_quote( $key ) .
+				'(.*?)<\/mw:editsection>/',
+				'\1 $' . htmlspecialchars( $tag[0]->getTex() ) . '\2</mw:editsection>', $text );
+			$text = str_replace( $key, $value, $text );
+		}
+		// This hook might be called multiple times. However one the tags are rendered the job is done.
+		self::$tags = [];
 		return true;
 	}
-
 	/**
 	 *
 	 * @global type $wgOut
@@ -358,16 +390,30 @@ class MathHooks {
 	 */
 	static function onEditPageBeforeEditToolbar( &$toolbar ) {
 		global $wgOut;
-		$wgOut->addModules( array( 'ext.math.editbutton.enabler' ) );
+		$wgOut->addModules( [ 'ext.math.editbutton.enabler' ] );
 	}
 
 	public static function registerExtension() {
 		global $wgDefaultUserOptions, $wgMathValidModes, $wgMathDisableTexFilter;
 		$wgMathValidModes = MathRenderer::getValidModes();
-		if ( $wgMathDisableTexFilter == true ){ //ensure backwards compatibility
-			$wgMathDisableTexFilter = 1;
+		if ( $wgMathDisableTexFilter === true ) { // ensure backwards compatibility
+			$wgMathDisableTexFilter = 'never';
 		}
 		$wgMathDisableTexFilter = MathRenderer::getDisableTexFilter();
 		$wgDefaultUserOptions['math'] = self::mathModeToString( $wgDefaultUserOptions['math'] );
 	}
+
+	/**
+	 * Callback function for the <ce> parser hook.
+	 *
+	 * @param $content (the LaTeX input)
+	 * @param $attributes
+	 * @param Parser $parser
+	 * @return array
+	 */
+	static function ceTagHook( $content, $attributes, $parser ) {
+		$attributes['chem'] = true;
+		return MathHooks::mathTagHook( '\ce{' . $content . '}', $attributes, $parser );
+	}
+
 }
