@@ -3,7 +3,13 @@ Given(/^I am editing a random page with source editor$/) do
 end
 
 When(/^I click Preview$/) do
-  on(EditPage).preview
+  on(EditPage).preview_element.when_present.click
+end
+
+When(/^I dismiss Welcome to Wikipedia popup$/) do
+  if env.lookup(:mediawiki_environment, default: nil) == 'beta'
+    on(EditPage).start_editing_element.click
+  end
 end
 
 When(/^I type (.+)$/) do |write_text|
